//  ANDY's AMAZING OPENGL MUSIC VISUAL THINGY

// #DEFINES & #INCLUDES	---------------------------------------------------------------------


//WINDOWS 
#include "StdAfx.h" 
#include <windows.h> 
#include "glut.h" 
#include <stdio.h>
#include <stdlib.h>
#include <iostream.h>
#include <time.h> 
#include <math.h> 
#include <string.h> 
#include <assert.h> 
#include "tga.h"
#include "Frontend.h"

#define PI 3.1415926535898
#define AVERAGING_FRAME_LENGTH 1200 //no frames over which data is averaged
#define CIRCLE_POINTS 60
#define NUM_BGDS 15

struct circle_listnode {
			float x_position;
			double y_position;
			int z_depth;
			float velocity[3];
			int age;
			struct circle_listnode *nextptr, *prevptr;
};
typedef struct circle_listnode CIRCLE_LISTNODE;

struct ripple_listnode {
			float x_position;
			double y_position;
			int z_depth;
			int age;
			float velocity;
			struct ripple_listnode *nextptr, *prevptr;
};
typedef struct ripple_listnode RIPPLE_LISTNODE;


// FUNCTION PROTOTYPES	----------------------------------------------------------------------

int DrawVisuals(void); 
void initTextures(void);
void init(void);
void display(void);
void reshape (int w, int h);
void specialUp (int key, int x, int y);
void keyUp (unsigned char key, int x, int y);
void specialDown (int key, int x, int y);
void keyDown (unsigned char key, int x, int y);
void idle(void);
int DrawVisuals(void);
void get_data_from_winamp();
void sort_data_into_eight_channels();
void average_the_data();
void update_running_totals();
void calculate_fps();
void push_CIRCLE_LISTNODE(CIRCLE_LISTNODE **, float, double, int, float, float, float);
void delete_CIRCLE_LISTNODE(CIRCLE_LISTNODE **, CIRCLE_LISTNODE **);
void push_RIPPLE_LISTNODE(RIPPLE_LISTNODE **, float, double, int);
void delete_RIPPLE_LISTNODE(RIPPLE_LISTNODE **, RIPPLE_LISTNODE **);
void test_vis();
void circle_vis();
void ripple_vis();
void bar_vis();


// VARIABLES	------------------------------------------------------------------------------
CIRCLE_LISTNODE *headptr = NULL;
RIPPLE_LISTNODE *head_ripple_ptr = NULL;

bool keys[128], specials[128];	// keyboard arrays
int winW = 600;					// window width 
int winH = 450;					// window height 
int first_time = 1;
int array_pos = 0;
int previous_data[12][AVERAGING_FRAME_LENGTH] = {0};		//8th one is data for all channels, 9,10,11, low,mid,high
int raw_data[288][2];
int eight_channels_mono[8];
int average_data[12] = {0};		//8th one is data for all channels, 9,10,11, low,mid,high
int average_above_average_data[8] = {0};
int this_frame_volume_average[4] = {0}; //0 is all ranges. 1,2&3 are low, mid and high range
int beat_per_channel[8];
int beat_frame[4] = {0};
int beat_counter[4][2] = {0};
int frame_counter=0;
int frames_per_second=25;
int current_time = 0;
int beat_roll_off[4]= {0};	//0 is all ranges. 1,2&3 are low, mid and high range
time_t mytime;
float circle_RGB = {1.0};
float base_RGB[3] = {0.0};
float secondary_RGB[3] = {1};
int colour_change_countdown = 0;
bool drift = FALSE;
int time_to_drift = 0;
int time_to_change_circles = 1;
bool rain = TRUE;
bool burst = FALSE;
int texture_mode = 0;
int bgd_mode = 0;
int texture_change_countdown = 0;
int flower_extra_radius = 0;
bool sketchy_camera_mode = FALSE;
bool bar_mode = TRUE;
int sketchy_camera_change_countdown = 0;
float bgd_depth;
float bgd_z_velocity;
double bgd_brightness_offset = 0;
int time_between_last_two_beats;
int time_since_beat_counter;
float bar_y_offset;
float circle_sin_values[CIRCLE_POINTS];
float circle_cos_values[CIRCLE_POINTS];
char filename[256];


// MAIN	--------------------------------------------------------------------------------------

int main(int argc, char** argv) 
{
	int i;

	srand(time(NULL));

	for(i=0; i<CIRCLE_POINTS; i++)
	{
		circle_sin_values[i] = sin(2*PI*i/CIRCLE_POINTS);
		circle_cos_values[i] = cos(2*PI*i/CIRCLE_POINTS);
	}

	glutInit(&argc, argv); 
	glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB);

	glutInitWindowSize (winW, winH); 
	glutInitWindowPosition (100, 100); 
	glutCreateWindow ("Andy's Winamp to OpenGL Audio Visual Muncher");

//	glutGameModeString("320x240:16@500");
//	glutEnterGameMode();

	glClearColor(0, 0, 0, 0);
	glClear(GL_COLOR_BUFFER_BIT);

	glutDisplayFunc(display); 
	glutReshapeFunc(reshape); 
	glutKeyboardFunc(keyDown); 
	glutSpecialFunc(specialDown);
	glutIdleFunc(idle); 
	glutKeyboardUpFunc(keyUp); 
	glutSpecialUpFunc(specialUp);

	init();
	glutMainLoop();
 
	return 0; 
} 


// OPENGL FUNCTIONS	------------------------------------------------------------------------

void init(void) 
{
	glClearColor(base_RGB[0], base_RGB[1], base_RGB[2], 0);
	glMatrixMode(GL_PROJECTION);
	glShadeModel (GL_SMOOTH);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glPolygonMode (GL_FRONT_AND_BACK, GL_FILL);
	loadTGA ("./textures/bgd_start.tga",  2);
	loadTGA ("./textures/bars.tga",  3);
	loadTGA ("./textures/circles.tga",  1);

//	glEnable(GL_LINE_SMOOTH);
} 
 
void display(void) 
{ 
//	UPDATE WINAMP DATA
	calculate_fps();
	get_data_from_winamp();
	sort_data_into_eight_channels();
	average_the_data();

//	CAMERA DATA THEN VISUALISE
	glPushMatrix ();
	if (sketchy_camera_mode == TRUE)	gluLookAt(rand()%360, rand()%360, rand()%360, 0, 0, 0, 0, 1, 0); 
	else		gluLookAt(0,0,250, 0, 0, 0, 0, 1, 0); 
		DrawVisuals(); 
	glPopMatrix(); 

if(first_time == 0 || array_pos==1)
{
	glutSwapBuffers();
}


	glFlush();
} 
 
void reshape (int w, int h) 
{  
	glViewport(0, 0, (GLsizei) w, (GLsizei) h); 
	glMatrixMode (GL_PROJECTION); 
	glLoadIdentity(); 
	gluPerspective(60.0, (GLfloat) w/(GLfloat) h, 1.0, 500.0); 
	glMatrixMode(GL_MODELVIEW); 
	glLoadIdentity();  
} 
 
 
//	keyboard up functions 
void specialUp (int key, int x, int y) 
{ 
	specials[key] = false; 
} 

void keyUp (unsigned char key, int x, int y) 
{ 
	keys[key] = false; 
} 
 
//	keyboard down functions 
void specialDown (int key, int x, int y) 
{ 
	specials[key] = true; 
} 
 
void keyDown (unsigned char key, int x, int y) 
{ 
	keys[key] = true; 
} 


void idle(void) 
{ 
	if (keys[27]) 
	{ 
//		if(glutGameModeGet(GLUT_GAME_MODE_ACTIVE))
//		{
//			glutLeaveGameMode();
//		}
		exit(0); 
	} 
	glutPostRedisplay(); 
} 


int DrawVisuals(void) 
{
	double fade;

if(first_time == 0)
{
	colour_change_countdown--;

	if((colour_change_countdown<(-(10*frames_per_second))) || (colour_change_countdown<0 && beat_frame[0] == 1))
	{
		// SET BASE COLOUR	
		base_RGB[0] = (rand()%100) * 0.01;
		base_RGB[1] = (rand()%100) * 0.01;
		base_RGB[2] = (rand()%100) * 0.01;

		// SET CIRCLES COLOUR
//		if(texture_mode!=1)

		if((((base_RGB[0]+base_RGB[1]+base_RGB[2])/3)>0.5))
		{
				circle_RGB=0.0;
		}
		else	circle_RGB=1.0;

		// SET NEW TEXTURES IF IT IS TIME
		texture_change_countdown--;

		if(texture_change_countdown<0)
		{
			// LOAD NEW CIRCLES TEXTURE
/*			texture_mode = (rand()%2);
			if(texture_mode<1)
			{
				flower_extra_radius=5;
				loadTGA ("./textures/circles_boom.tga",  1);
			}
			else if(texture_mode==1)
			{
				flower_extra_radius=5;
				loadTGA ("./textures/circles_flower.tga",  1);
				loadTGA ("./textures/bgd_hearts.tga",  2);
				circle_RGB=1.0;
			}
*/
			//	LOAD NEW BACKGROUND TEXTURE, DEFAULT BGD IF SKETCHY CAM IS ACTIVE

			if(sketchy_camera_mode == TRUE)
			{
				loadTGA ("./textures/bgd_1.tga",  2);
			}
			else  //if(texture_mode!=1)
			{
				bgd_mode = ((rand()%(NUM_BGDS-1))+2);
				sprintf(filename, "./textures/bgd_%d.tga", bgd_mode);
				loadTGA (filename,  2);
			}

			texture_change_countdown = (2+(rand()%8));

			// SET RIPPLES OR BARS
			if(rand()%2 < 0.5) bar_mode = FALSE;
			else bar_mode = TRUE;
		}

		// SET DRIFT MODE
		time_to_drift--;
		if(time_to_drift<0)
		{
			time_to_drift= rand()%9; //DRIFT FOR CIRCLES
			if(time_to_drift<=2.0)				drift=TRUE;
				else if(time_to_drift>2.0)		drift=FALSE;
			time_to_drift = rand()%3;
		}

		// CHANGE CIRCLE MODE IF IT IS TIME
		time_to_change_circles--;
		if(time_to_change_circles<0)
		{
			if(burst == TRUE)
			{
				burst = FALSE;
				rain  = TRUE;
			}
			else
			{
				burst = TRUE;
				rain  = FALSE;
			}
			if(rain==TRUE) time_to_change_circles = rand()%7;
			else time_to_change_circles = rand()%3;
		}

		// SET SKETCHY CAMERA MODE IF IT IS TIME
		sketchy_camera_change_countdown--;
		if(sketchy_camera_change_countdown<0)
		{
			if(sketchy_camera_mode == TRUE)
			{
				sketchy_camera_mode = FALSE;
				sketchy_camera_change_countdown = rand()%30;
				texture_change_countdown=0;
			}
			else
			{
				sketchy_camera_mode = TRUE;
				sketchy_camera_change_countdown = rand()%14;
				loadTGA ("./textures/bgd_1.tga",  2);
			}
		}

		//	SET COLOUR CHANGE COUNTDOWN
		if (rand()%11 > 9 && sketchy_camera_mode == FALSE)
			colour_change_countdown = (4*frames_per_second)+(rand()%(int)(8*frames_per_second));
		else colour_change_countdown = 0;
	}

//	FADE OVER POLYGON
	fade = (6.0/frames_per_second);	//SET FADE (OPACITY) VALUE

	// SET BGD DEPTH, Z-VELOCITY & MAKE MORE OPAQUE ON A BEAT FRAME
	if(beat_counter[0][0]==1)
	{
		bgd_depth = this_frame_volume_average[0];
		bgd_z_velocity = (float)this_frame_volume_average[0]/(time_between_last_two_beats-beat_roll_off[0]);
		if(beat_frame[0] == 1 && sketchy_camera_mode == FALSE)	fade *= 8;
	}

	// UPDATE Z-DEPTH OF BGND
	bgd_depth -= bgd_z_velocity;
	if(bgd_depth<(average_data[12]/AVERAGING_FRAME_LENGTH)) bgd_depth=(this_frame_volume_average[0]);

	// SET BGND BRIGHTNESS MULTIPLIER
	bgd_brightness_offset = 1.0;
	if(sketchy_camera_mode == TRUE) bgd_brightness_offset = 0.7;
	bgd_brightness_offset += (float)((((this_frame_volume_average[0]/2.0)-(average_data[8]/AVERAGING_FRAME_LENGTH))/(3*(average_data[8]/AVERAGING_FRAME_LENGTH))+0.0001));
	if(this_frame_volume_average[0]==0) bgd_brightness_offset = 0.0;

	// SET BGD COLOUR
	glColor4f (base_RGB[0]*bgd_brightness_offset,base_RGB[1]*bgd_brightness_offset,base_RGB[2]*bgd_brightness_offset,fade);

	bgd_depth -= 10;

	// DISPLAY BGND
	glEnable (GL_TEXTURE_2D); // enable texture mapping
	glBindTexture (GL_TEXTURE_2D, 2);
	glBegin(GL_POLYGON);
		glTexCoord2f(0.0,0.0);
		glVertex3f(-200.0, -150.0, (float)bgd_depth/1.7);
		glTexCoord2f(1.0,0.0);
		glVertex3f( 200.0, -150.0, (float)bgd_depth/1.7);
		glTexCoord2f(1.0,1.0);
		glVertex3f( 200.0,  150.0, (float)bgd_depth/1.7);
		glTexCoord2f(0.0,1.0);
		glVertex3f(-200.0,  150.0, (float)bgd_depth/1.7);
	glEnd();
	glDisable (GL_TEXTURE_2D);

// ORDER IMPORTANT FOR ORDER DRAWN
	//	test_vis();
	if (sketchy_camera_mode == FALSE && bar_mode == FALSE) ripple_vis();
	circle_vis();
	if (sketchy_camera_mode == FALSE && bar_mode == TRUE) bar_vis();
}
else if(array_pos==1)
{
//	LOADING SCREEN
	if(beat_frame[0])
	{
		bgd_depth = this_frame_volume_average[0];
		bgd_z_velocity = (float)this_frame_volume_average[0]/time_between_last_two_beats;
	}
	bgd_depth -= bgd_z_velocity;
	if(bgd_depth<(average_data[12]/AVERAGING_FRAME_LENGTH)) bgd_depth=this_frame_volume_average[0];

	fade = (6.0/frames_per_second);
	glColor4f (1.0,1.0,1.0,fade);

	bgd_depth -= 10;

	glEnable (GL_TEXTURE_2D); // enable texture mapping
	glBindTexture (GL_TEXTURE_2D, 2);
	glBegin(GL_POLYGON);
		glTexCoord2f(0.0,0.0);
		glVertex3f(-200.0, -150.0, 0.0);
		glTexCoord2f(1.0,0.0);
		glVertex3f( 200.0, -150.0, 0.0);
		glTexCoord2f(1.0,1.0);
		glVertex3f( 200.0,  150.0, 0.0);
		glTexCoord2f(0.0,1.0);
		glVertex3f(-200.0,  150.0, 0.0);
	glEnd();
	glDisable (GL_TEXTURE_2D);

}

	return 1; 
}

void test_vis()
{
	int x=0;
	GLfloat offset, offset2=30;

//	DRAW BEAT_PER_CHANNEL
	glColor4f (0.0,0.0,1.0,1.0);
	offset = 1.0;
	for(x=0;x<8;x++)
	{
		if(beat_per_channel[x]==1)
		{
			glBegin(GL_POLYGON);
				glVertex3f(offset-180.0, -144.0, 0.0);
				glVertex3f(offset-142.0, -144.0, 0.0);
				glVertex3f(offset-142.0,  144.0, 0.0);
				glVertex3f(offset-180.0,  144.0, 0.0);
			glEnd();
		}
		offset+=40.0;
	}


//	DRAW SPECTRUM ANALYSER
	glColor4f (0.3,1.0,0.5,1.0);
	offset = 4.4;
	for(x=0;x<8;x++)
	{
		glBegin(GL_POLYGON);
			glVertex3f(offset-180.0, -144.0, 0.0);
			glVertex3f(offset-176.4, -144.0, 0.0);
			glVertex3f(offset-176.4, eight_channels_mono[x]-144.0, 0.0);
			glVertex3f(offset-180.0, eight_channels_mono[x]-144.0, 0.0);
		glEnd();
		offset+=4.4;
	}

//	DRAW BEAT COUNTERS
	for(x=0;x<4;x++)
	{
		if(beat_roll_off[x]<(int)(frames_per_second/8))
		{
			offset = 24.0 + (20*beat_counter[x][1]);
			offset2 = 30.0*x;
			glColor4f (1.0,0.0,0.0,1.0);
			glEnable (GL_TEXTURE_2D); // enable texture mapping
				glBindTexture (GL_TEXTURE_2D, 1);
				glBegin(GL_POLYGON);
					glTexCoord2f(0.0,0.0); glVertex3f(offset-180.0, offset2-94.0, 0.0);
					glTexCoord2f(0.0,1.0); glVertex3f(offset-164.0, offset2-94.0, 0.0);
					glTexCoord2f(1.0,1.0); glVertex3f(offset-164.0, offset2-69.0, 0.0);
					glTexCoord2f(1.0,0.0); glVertex3f(offset-180.0, offset2-69.0, 0.0);
				glEnd();
			glDisable (GL_TEXTURE_2D);
		}
	}
}

void circle_vis()
{
	int x, i, amount = 5;
	double offset, offset2;
	float plus_x, plus_y, plus_z;
	CIRCLE_LISTNODE *currentptr, *deleteptr;

//RAIN   ------------------------------- RAIN ----------------------------------- RAIN
	if(rain==TRUE)
	{
		// CREATE NEW DROPS
		for(x=0;x<4;x++)
		{
			if(beat_frame[x]==1)
			{
				for(i=0;i<amount;i++)
				{
					push_CIRCLE_LISTNODE(&headptr, (((double)(rand()%400))-200.0), (200.0)+(float)(rand()%15),(50-rand()%100), 1.0/(float)(rand()%10), 1.0/(float)(rand()%10), 1.0/(float)(rand()%10));
				}
			}

			if(beat_frame[0]==1)	amount = 1 + rand()%4;
				else amount = rand()%2;
		}

		// UPDATE Y POSTITION
		offset = (double)(80.0)/(double)(frames_per_second);
		currentptr =  headptr;

		while(currentptr!=NULL)
		{
		//	update position
			currentptr->y_position -= offset;
			currentptr = currentptr->nextptr;
		}

	
		currentptr =  headptr;

		//	DELETE IF OUT OF RANGE OF SCREEN
		while(currentptr!=NULL)
		{
			if(currentptr->y_position < (-300.0))
			{
				deleteptr = currentptr;
				currentptr = currentptr->nextptr;
				delete_CIRCLE_LISTNODE(&deleteptr, &headptr);
			}
			else
			{
				currentptr = currentptr->nextptr;
			}
		}

		// SET THE WIDTH OF THE DROPS
		currentptr = headptr;
//		offset = this_frame_volume_average[0]/20;
		offset = this_frame_volume_average[0]/10;
		if(drift==TRUE)  offset2 = this_frame_volume_average[0]/2.0;
		else offset2 = 0.0;

		if(sketchy_camera_mode == TRUE)
		{
			if(beat_counter[0][0] == 1) offset = 10+(offset/7);
			else if	(beat_counter[1][0] == 1 || 
					 beat_counter[2][0] == 1 ||
					 beat_counter[3][0] == 1)	offset = 1+(offset/6);
			else offset = offset/6;
		}

		// DISPLAY THE DROPS
		while(currentptr!=NULL)
		{
			glEnable (GL_TEXTURE_2D); // enable texture mapping
				glBindTexture (GL_TEXTURE_2D, 1);


			if(drift==TRUE)
			{
				glColor4f (circle_RGB,circle_RGB,circle_RGB,1.0);
					glBegin(GL_POLYGON);
						glTexCoord2f(0.0,0.0);
						glVertex3f(currentptr->x_position-offset+offset2-flower_extra_radius,
							currentptr->y_position-flower_extra_radius,		currentptr->z_depth+offset2);		

						glTexCoord2f(1.0,0.0);
						glVertex3f(currentptr->x_position+offset+offset2+flower_extra_radius,
							currentptr->y_position-flower_extra_radius,		currentptr->z_depth+offset2);

						glTexCoord2f(1.0,1.0);
						glVertex3f(currentptr->x_position+offset+offset2+flower_extra_radius,
							currentptr->y_position+offset+flower_extra_radius, currentptr->z_depth+offset2);

						glTexCoord2f(0.0,1.0);
						glVertex3f(currentptr->x_position-offset+offset2-flower_extra_radius,
							currentptr->y_position+offset+flower_extra_radius, currentptr->z_depth+offset2);
					glEnd();
				glDisable (GL_TEXTURE_2D);
			}
			else
			{
				glColor4f (circle_RGB,circle_RGB,circle_RGB,0.8);
					glBegin(GL_POLYGON);
						glTexCoord2f(0.0,0.0);
						glVertex3f(currentptr->x_position-offset+(offset2*40)-flower_extra_radius,
							currentptr->y_position-flower_extra_radius,		currentptr->z_depth*(offset2*2));		

						glTexCoord2f(1.0,0.0);
						glVertex3f(currentptr->x_position+offset+(offset2*60)+flower_extra_radius,
							currentptr->y_position-flower_extra_radius,		currentptr->z_depth*(offset2*2));

						glTexCoord2f(1.0,1.0);
						glVertex3f(currentptr->x_position+offset+(offset2*60)+flower_extra_radius,
							currentptr->y_position+offset+(offset2*10)+flower_extra_radius, currentptr->z_depth*(offset2*2));

						glTexCoord2f(0.0,1.0);
						glVertex3f(currentptr->x_position-offset+(offset2*40)-flower_extra_radius,
							currentptr->y_position+offset+(offset2*10)+flower_extra_radius, currentptr->z_depth*(offset2*2));
					glEnd();
				glDisable (GL_TEXTURE_2D);
			}

			currentptr = currentptr->nextptr;
		}
	}

//	BURST   ------------------------------- BURST ----------------------------------- BURST
	if(burst==TRUE)
	{
		// CALCULATE HOW MANY NEW DROPS TO RELEASE THIS FRAME
		for(x=0;x<4;x++)
		{
			if(beat_frame[x]==1)
			{
				for(i=0;i<amount;i++)
				{
						if((rand()%2)==0) plus_x = -1.0;
							else plus_x = 1.0;
						if((rand()%2)==0) plus_y = -1.0;
							else plus_y = 1.0;
						if((rand()%2)==0) plus_z = -1.0;
							else plus_z = 1.0;
						push_CIRCLE_LISTNODE(&headptr, 0.0, 0.0, (100-rand()%200), (1.0/(float)(rand()%10))*plus_x,
												(1.0/(float)(rand()%10))*plus_y, 0.0);
				}
			}

			if(beat_frame[0]==1)	amount = 15+rand()%30;
				else amount = rand()%10;
		}

		//UPDATE VELOCITIES
		offset = (double)(300.0)/(double)(frames_per_second);
		currentptr =  headptr;

		while(currentptr!=NULL)
		{
		//	update position and age
			currentptr->age++;

			//	MAGNETIC EFFECT
			if(currentptr->velocity[0]>0)	currentptr->velocity[0]+=(0.2/(float)(frames_per_second));
				else						currentptr->velocity[0]-=(0.2/(float)(frames_per_second));

			if(currentptr->velocity[1]>0)	currentptr->velocity[1]-=
												(2.0*currentptr->velocity[1]*(0.5/(float)(frames_per_second)));
				else						currentptr->velocity[1]+=
												((-2.0*currentptr->velocity[1])*(0.5/(float)(frames_per_second)));
			
			currentptr->x_position += offset*currentptr->velocity[0];
			currentptr->y_position += (offset*currentptr->velocity[1]);
			currentptr->z_depth += offset*currentptr->velocity[2];
			currentptr = currentptr->nextptr;
		}

		currentptr =  headptr;

		//	DELETE IF OUT OF RANGE OF SCREEN
		while(currentptr!=NULL)
		{
			if(	currentptr->y_position < (-300.0) || currentptr->y_position > 300.0  ||
				currentptr->x_position < (-300.0) || currentptr->x_position > 300.0  ||
				currentptr->z_depth < (-300) || currentptr->z_depth > 300)
			{
				deleteptr = currentptr;
				currentptr = currentptr->nextptr;
				delete_CIRCLE_LISTNODE(&deleteptr, &headptr);
			}
			else
			{
				currentptr = currentptr->nextptr;
			}
		}

		currentptr = headptr;
//		offset = this_frame_volume_average[0]/20;
		offset = this_frame_volume_average[0]/10;

		// SPECIAL TWEAKS FOR DIFFERENT MODES
		if(drift==TRUE)  offset2 = this_frame_volume_average[0]/2.0;
		else offset2 = 0.0;

		if(sketchy_camera_mode == TRUE)
		{
			if(beat_counter[0][0] == 1) offset = 10+(offset/7);
			else if	(beat_counter[1][0] == 1 || 
					 beat_counter[2][0] == 1 ||
					 beat_counter[3][0] == 1)	offset = 1+(offset/6);
			else offset = offset/6;
		}

		//DISPLAY THE EXPLOSION
		while(currentptr!=NULL)
		{
			glColor4f (circle_RGB,circle_RGB,circle_RGB,
							(float)(0.1/((float)(currentptr->age)/(float)(frames_per_second))));
		

			glEnable (GL_TEXTURE_2D); // enable texture mapping
				glBindTexture (GL_TEXTURE_2D, 1);

				glBegin(GL_POLYGON);
					glTexCoord2f(0.0,0.0);
					glVertex3f(currentptr->x_position-offset-1-flower_extra_radius,
									currentptr->y_position-1-flower_extra_radius,currentptr->z_depth+offset2);	

					glTexCoord2f(1.0,0.0);
					glVertex3f(currentptr->x_position+offset+1+flower_extra_radius,
									currentptr->y_position-1-flower_extra_radius,currentptr->z_depth+offset2);

					glTexCoord2f(1.0,1.0);
					glVertex3f(currentptr->x_position+offset+1+flower_extra_radius,	
								currentptr->y_position+1+offset+flower_extra_radius,currentptr->z_depth+offset2);

					glTexCoord2f(0.0,1.0);
					glVertex3f(currentptr->x_position-offset-1-flower_extra_radius,	
								currentptr->y_position+1+offset+flower_extra_radius, currentptr->z_depth+offset2);
				glEnd();
			glDisable (GL_TEXTURE_2D);

			currentptr = currentptr->nextptr;
		}
	}
}

void push_CIRCLE_LISTNODE(CIRCLE_LISTNODE **headptr, float x_position, double y_position, int z_depth, float vel_x, float vel_y, float vel_z)
{
	CIRCLE_LISTNODE *newptr;
		
	newptr = new CIRCLE_LISTNODE;
	
	if(newptr!=NULL)	//is space available?
	{
		newptr->x_position = x_position;
		newptr->y_position = y_position;
		newptr->z_depth = z_depth;
		newptr->velocity[0] = vel_x;
		newptr->velocity[1] = vel_y;
		newptr->velocity[2] = vel_z;
		newptr->prevptr = NULL;
		newptr->nextptr = NULL;
		newptr->age=0;

		//insert new_circle at head
		newptr->nextptr = *headptr;
		*headptr=newptr;
		if((*headptr)->nextptr != NULL)	((*headptr)->nextptr)->prevptr = *headptr;
	}
	else printf("no memory left for another circle\n");
	(*headptr)->prevptr=NULL;
}

void delete_CIRCLE_LISTNODE(CIRCLE_LISTNODE **deleteptr, CIRCLE_LISTNODE **headptr)
{
	if((*deleteptr)->prevptr == NULL)  //if head
	{
		*headptr = (*deleteptr)->nextptr;
		if (*headptr!=NULL) (*headptr)->prevptr = NULL;
		delete (*deleteptr);
	}
	else
	{
		((*deleteptr)->prevptr)->nextptr = (*deleteptr)->nextptr;
		if((*deleteptr)->nextptr != NULL)
			((*deleteptr)->nextptr)->prevptr = (*deleteptr)->prevptr;
		delete (*deleteptr);
	}
	if(*headptr!=NULL) (*headptr)->prevptr=NULL;
}

void bar_vis()
{
	int x=0;
	GLfloat offset;

//	DRAW 8 BARS
	glColor4f (base_RGB[0],base_RGB[1],base_RGB[2],0.5);
	offset = 1.0;
	for(x=0;x<8;x++)
	{
		if(beat_per_channel[x]==1)
		{
			glEnable (GL_TEXTURE_2D); // enable texture mapping
			glBindTexture (GL_TEXTURE_2D, 3);

			bar_y_offset = (float)(((x/2)+1)*((this_frame_volume_average[x]-(average_data[x]/AVERAGING_FRAME_LENGTH))));

			glBegin(GL_POLYGON);
				glTexCoord2f(0.0,0.0);
				glVertex3f(offset-202.0, -144.0+(bar_y_offset/4), (bar_y_offset/(8-x)));
				glTexCoord2f(1.0,0.0);
				glVertex3f(offset-150.0, -144.0+(bar_y_offset/4), (bar_y_offset/(8-x)));
				glTexCoord2f(1.0,1.0);
				glVertex3f(offset-150.0,  144.0-(bar_y_offset/4), (bar_y_offset/(8-x)));
				glTexCoord2f(0.0,1.0);
				glVertex3f(offset-202.0,  144.0-(bar_y_offset/4), (bar_y_offset/(8-x)));
			glEnd();
			glDisable (GL_TEXTURE_2D);
		}
		offset+=50.0;
	}
}


void ripple_vis()
{
	int x, i, line_width;
	float opacity;
	RIPPLE_LISTNODE *currentptr, *deleteptr;

	// SET BLEND FUNCTION TO CREATE GLOW WHERE LINES OVERLAP
	glBlendFunc(GL_SRC_ALPHA, GL_ONE);

	// UPDATE LINE WIDTH
	line_width = (int)((2*bgd_depth)+2);
	if(line_width<5) line_width=5;
	glLineWidth(line_width);

	// CREATE NEW RIPPLES
	if(beat_frame[0]==1) push_RIPPLE_LISTNODE(&head_ripple_ptr, (rand()%400)-200,rand()%320-160,0);

	currentptr =  head_ripple_ptr;	

	// DELETE RIPPLES IF OUT OF RANGE
	while(currentptr!=NULL)
	{
		//	update age
		currentptr->age++;

		if((currentptr->age*currentptr->velocity) > 500)
		{
			deleteptr = currentptr;
			currentptr = currentptr->nextptr;
			delete_RIPPLE_LISTNODE(&deleteptr, &head_ripple_ptr);
		}
		else
		{
			currentptr = currentptr->nextptr;
		}
	}

	// DISPLAY RIPPLES
	currentptr = head_ripple_ptr;

	while(currentptr!=NULL)
	{
		opacity = (float)((16-(12/((80*this_frame_volume_average[0])+80.0))) / frames_per_second);
		glColor4f (base_RGB[0],base_RGB[1],base_RGB[2],opacity);

		for(x=0;x<1;x++)
		{
				glBegin(GL_LINE_STRIP);
				for(i=0; i<CIRCLE_POINTS; i++)
				{
					glVertex2f((float)((circle_sin_values[i]*currentptr->age*currentptr->velocity) + 
											currentptr->x_position),
							   (float)((circle_cos_values[i]*currentptr->age*currentptr->velocity) + 
											currentptr->y_position));
				}
					glVertex2f((float)((circle_sin_values[0]*currentptr->age*currentptr->velocity) + 
											currentptr->x_position),
							   (float)((circle_cos_values[0]*currentptr->age*currentptr->velocity) + 
											currentptr->y_position));
				glEnd();
		}

			currentptr = currentptr->nextptr;
	}

	// REVERT BLENDING MODE
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
}

void push_RIPPLE_LISTNODE(RIPPLE_LISTNODE **head_ripple_ptr, float x_position, double y_position, int z_depth)
{
	RIPPLE_LISTNODE *newptr;
		
	newptr = new RIPPLE_LISTNODE;
	
	if(newptr!=NULL)	//is space available?
	{
		newptr->x_position = x_position;
		newptr->y_position = y_position;
		newptr->z_depth = z_depth;
		newptr->prevptr = NULL;
		newptr->nextptr = NULL;
		newptr->age=0;
		newptr->velocity = 80.0/((float)frames_per_second);

		//insert new_circle at head
		newptr->nextptr = *head_ripple_ptr;
		*head_ripple_ptr=newptr;
		if((*head_ripple_ptr)->nextptr != NULL)	((*head_ripple_ptr)->nextptr)->prevptr = *head_ripple_ptr;
	}
	else printf("no memory left for another circle\n");
	(*head_ripple_ptr)->prevptr=NULL;
}

void delete_RIPPLE_LISTNODE(RIPPLE_LISTNODE **deleteptr, RIPPLE_LISTNODE **head_ripple_ptr)
{
	if((*deleteptr)->prevptr == NULL)  //if head
	{
		*head_ripple_ptr = (*deleteptr)->nextptr;
		if (*head_ripple_ptr!=NULL) (*head_ripple_ptr)->prevptr = NULL;
		delete (*deleteptr);
	}
	else
	{
		((*deleteptr)->prevptr)->nextptr = (*deleteptr)->nextptr;
		if((*deleteptr)->nextptr != NULL)
			((*deleteptr)->nextptr)->prevptr = (*deleteptr)->prevptr;
		delete (*deleteptr);
	}
	if(*head_ripple_ptr!=NULL) (*head_ripple_ptr)->prevptr=NULL;
}


// NON_VISUAL FUNCTIONS	---------------------------------------------------------------------

void get_data_from_winamp()
{
	int x, y;
	FILE *data_filePtr;
	
	if ((data_filePtr = fopen("C:/data.txt", "rb")) == NULL)
			printf("file could not be opened\n");
	else
	{
		for (y = 0; y < 2; y ++)
		{
			for (x = 0; x < 288; x ++)
			{
				//SCAN FROM FILE
				fscanf(data_filePtr, "%d", &raw_data[x][y]);
			}
		}
	//CLOSE FILE
		fclose(data_filePtr);
	}
}

void sort_data_into_eight_channels()
{
	int x=0, val=0;

// step through every eighth of the data, sum and average it (divide by 36), assign to array
	while(x<36)
	{
		val+=raw_data[x][0];
		x++;
	}
	eight_channels_mono[0]=(val/36);
	val=0;
	while(x<72)
	{
		val+=raw_data[x][0];
		x++;
	}
	eight_channels_mono[1]=(val/36);
	val=0;
	while(x<108)
	{
		val+=raw_data[x][0];
		x++;
	}
	eight_channels_mono[2]=(val/36);
	val=0;
	while(x<144)
	{
		val+=raw_data[x][0];
		x++;
	}
	eight_channels_mono[3]=(val/36);
	val=0;
	while(x<180)
	{
		val+=raw_data[x][0];
		x++;
	}
	eight_channels_mono[4]=(val/36);
	val=0;
	while(x<216)
	{
		val+=raw_data[x][0];
		x++;
	}
	eight_channels_mono[5]=(val/36);
	val=0;
	while(x<252)
	{
		val+=raw_data[x][0];
		x++;
	}
	eight_channels_mono[6]=(val/36);
	val=0;
	while(x<288)
	{
		val+=raw_data[x][0];
		x++;
	}
	eight_channels_mono[7]=(val/36);
}

void average_the_data()
{
	int x;

	array_pos++;
	if(array_pos==AVERAGING_FRAME_LENGTH)
	{
		array_pos = 0;
		first_time = 0;
	}
	
	for(x=0;x<4;x++)
	{	
		this_frame_volume_average[x] = 0;
	}

	for(x=0;x<8;x++)
	{
		beat_per_channel[x] = 0;

		average_data[x] -= previous_data[x][array_pos];
		previous_data[x][array_pos] = eight_channels_mono[x];
		average_data[x] += eight_channels_mono[x];
		
		// average the volume_average_per band of frequency
		this_frame_volume_average[0] += eight_channels_mono[x];

		if(x<3)	this_frame_volume_average[1] += eight_channels_mono[x];
			else if(x<6) this_frame_volume_average[2] += eight_channels_mono[x];
				else this_frame_volume_average[3] += eight_channels_mono[x];

		if((eight_channels_mono[x]/2.5) > (average_data[x]/AVERAGING_FRAME_LENGTH))
		{	
			//	Work out if each channel is on a beat
				beat_per_channel[x] = 1;
		}
	}

	time_since_beat_counter++;

	for(x=0;x<4;x++)
	{
		beat_frame[x]=0;

		average_data[x+8] -= previous_data[x+8][array_pos];
		previous_data[x+8][array_pos] = this_frame_volume_average[x];
		average_data[x+8] += this_frame_volume_average[x];

		beat_roll_off[x]++;
		beat_counter[x][0] = 0;

		if((this_frame_volume_average[x]/2.5) > (average_data[x+8]/AVERAGING_FRAME_LENGTH))
		{
			beat_counter[x][0] = 1;
			if(beat_roll_off[x]>(int)(frames_per_second/8))
			{
				beat_counter[x][1] += 1;
				beat_frame[x]=1;	//this frame's the first of a beat
				time_between_last_two_beats = time_since_beat_counter;
				time_since_beat_counter = 0;
			}
			beat_roll_off[x] = 0;
			if(beat_counter[x][1] > 4) beat_counter[x][1] = 1;
		}
	}
}

void calculate_fps()
{
	frame_counter++;
	time(&mytime);
	if (mytime>current_time)
	{
		frames_per_second = frame_counter;
		current_time = mytime;
		frame_counter=0;
//		printf("fps: %d\n", frames_per_second);
	}
}